require "import"
import "android.widget.*"
import "android.view.*"
import "android.text.*"
import "android.content.*"
import "android.speech.RecognizerIntent"
import "android.speech.SpeechRecognizer"
import "android.app.AlertDialog"

-- Activity title
activity.setTitle("Text Content")

-- Helper function to show toast messages
function showToast(message)
    Toast.makeText(activity, message, Toast.LENGTH_SHORT).show()
end

-- Speech recognizer and toggle state variables
local speechRecognizer
local recognizerListener
local hindiSpeechActive = false
local englishSpeechActive = false

-- Function to stop transcription
local function stopTranscription()
    if speechRecognizer then
        speechRecognizer.stopListening()
        speechRecognizer.destroy()
        speechRecognizer = nil
        showToast("Transcription stopped.")
    end
end

-- Function to handle live or normal transcription
local function startTranscription(language, mode)
    local recordIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recordIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recordIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, language)
    recordIntent.putExtra(RecognizerIntent.EXTRA_PARTIAL_RESULTS, mode == "live")

    speechRecognizer = SpeechRecognizer.createSpeechRecognizer(activity)
    recognizerListener = {
        onResults = function(results)
            local finalText = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION).get(0)
            if textEditor then
                textEditor.append(finalText .. " ")
            end
        end,
        onPartialResults = function(results)
            if mode == "live" then
                local partialText = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION).get(0)
                if textEditor then
                    textEditor.setText(partialText)
                    textEditor.setSelection(textEditor.getText().length()) -- Keep the cursor at the end
                end
            end
        end,
        onError = function(errorCode)
            showToast("Speech recognition error: " .. errorCode)
            stopTranscription()
        end
    }
    speechRecognizer.setRecognitionListener(recognizerListener)
    speechRecognizer.startListening(recordIntent)
    showToast(language .. " transcription (" .. mode .. ") started.")
end

-- Function to toggle transcription mode
local function toggleTranscriptionMode(language)
    local options = {"Live Transcription", "Normal Transcription"}
    local builder = AlertDialog.Builder(activity)
    builder.setTitle("Choose Transcription Mode")
    builder.setItems(options, function(dialog, which)
        local mode = which == 0 and "live" or "normal"
        if language == "hi-IN" then
            hindiSpeechActive = true
        else
            englishSpeechActive = true
        end
        startTranscription(language, mode)
    end)
    builder.setNegativeButton("Cancel", function(dialog) dialog.dismiss() end)
    builder.show()
end

-- Text analysis function
local function showTextAnalysis()
    local text = textEditor.getText().toString()

    -- Calculate metrics
    local characterCount = string.len(text)
    local wordCount = 0
    for _ in string.gmatch(text, "%S+") do
        wordCount = wordCount + 1
    end

    local sentenceCount = 0
    for _ in string.gmatch(text, "[^%.!?]+[%.!?]") do
        sentenceCount = sentenceCount + 1
    end

    local lineCount = 0
    for _ in string.gmatch(text, "[^\n]*\n?") do
        lineCount = lineCount + 1
    end

    local paragraphCount = 0
    for paragraph in string.gmatch(text, "[^\n]+") do
        if string.match(paragraph, "%S") then
            paragraphCount = paragraphCount + 1
        end
    end

    -- Create list of results
    local results = {
        "Characters: " .. characterCount,
        "Words: " .. wordCount,
        "Sentences: " .. sentenceCount,
        "Lines: " .. lineCount,
        "Paragraphs: " .. paragraphCount
    }

    -- Create ListView
    local listView = ListView(activity)
    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, String(results))
    listView.setAdapter(adapter)

    -- Show in a new dialog
    local builder = AlertDialog.Builder(activity)
    builder.setTitle("Text Analysis")
    builder.setView(listView)
    builder.setPositiveButton("OK", nil)
    local dialog = builder.create()
    dialog.show()
end

-- Define main layout
mainLayout = {
    LinearLayout,
    orientation = "vertical",
    layout_width = "fill",
    layout_height = "fill",
    {
        EditText,
        id = "textEditor",
        layout_width = "match_parent",
        layout_height = "0dp",
        layout_weight = 1,
        hint = "Write your text here...",
        textSize = "16sp",
        inputType = InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_FLAG_MULTI_LINE,
        gravity = "top",
    },
    {
        LinearLayout,
        orientation = "horizontal",
        layout_width = "match_parent",
        layout_height = "wrap_content",
        gravity = "center",
        {
            Button,
            id = "saveButton",
            layout_width = "0dp",
            layout_height = "wrap_content",
            layout_weight = 1,
            text = "Save Text",
        },
        {
            Button,
            id = "shareButton",
            layout_width = "0dp",
            layout_height = "wrap_content",
            layout_weight = 1,
            text = "Share Text",
        },
        {
            Button,
            id = "copyButton",
            layout_width = "0dp",
            layout_height = "wrap_content",
            layout_weight = 1,
            text = "Copy Text",
        },
        {
            Button,
            id = "pasteButton",
            layout_width = "0dp",
            layout_height = "wrap_content",
            layout_weight = 1,
            text = "Paste Text",
        },
        {
            Button,
            id = "cutButton",
            layout_width = "0dp",
            layout_height = "wrap_content",
            layout_weight = 1,
            text = "Cut Text",
        },
    },
    {
        LinearLayout,
        orientation = "horizontal",
        layout_width = "match_parent",
        layout_height = "wrap_content",
        gravity = "center",
        {
            Button,
            id = "hindiSpeechButton",
            layout_width = "0dp",
            layout_height = "wrap_content",
            layout_weight = 1,
            text = "Hindi Speech Typing",
        },
        {
            Button,
            id = "englishSpeechButton",
            layout_width = "0dp",
            layout_height = "wrap_content",
            layout_weight = 1,
            text = "English Speech Typing",
        },
        {
            Button,
            id = "analyzeTextButton",
            layout_width = "0dp",
            layout_height = "wrap_content",
            layout_weight = 1,
            text = "Analyze Text",
        },
    },
}

-- Set content view
activity.setContentView(loadlayout(mainLayout))

-- Button functionalities
saveButton.onClick = function()
    local text = textEditor.getText()
    if text == nil or text.length() == 0 then
        showToast("Text editor is empty!")
    else
        local textStr = text.toString()
        local path = activity.getFilesDir().toString() .. "/saved_text.txt"
        local file = io.open(path, "w")
        if file then
            file:write(textStr)
            file:close()
            showToast("Text saved to: " .. path)
        else
            showToast("Error saving text!")
        end
    end
end

shareButton.onClick = function()
    local text = textEditor.getText()
    if text == nil or text.length() == 0 then
        showToast("Text editor is empty!")
    else
        local textStr = text.toString()
        local intent = Intent(Intent.ACTION_SEND)
        intent.setType("text/plain")
        intent.putExtra(Intent.EXTRA_TEXT, textStr)
        activity.startActivity(Intent.createChooser(intent, "Share Text via"))
    end
end

copyButton.onClick = function()
    local text = textEditor.getText()
    if text == nil or text.length() == 0 then
        showToast("No text to copy!")
    else
        local textStr = text.toString()
        local clipboard = activity.getSystemService(Context.CLIPBOARD_SERVICE)
        local clip = ClipData.newPlainText("copiedText", textStr)
        clipboard.setPrimaryClip(clip)
        showToast("Text copied to clipboard!")
    end
end

pasteButton.onClick = function()
    local clipboard = activity.getSystemService(Context.CLIPBOARD_SERVICE)
    local clip = clipboard.getPrimaryClip()
    if clip == nil or clip.getItemCount() == 0 then
        showToast("No text to paste!")
    else
        local textToPaste = clip.getItemAt(0).getText()
        if textToPaste == nil then
            showToast("Clipboard is empty!")
        else
            textEditor.setText(textToPaste)
            showToast("Text pasted from clipboard!")
        end
    end
end

cutButton.onClick = function()
    local text = textEditor.getText()
    if text == nil or text.length() == 0 then
        showToast("No text to cut!")
    else
        local textStr = text.toString()
        local clipboard = activity.getSystemService(Context.CLIPBOARD_SERVICE)
        local clip = ClipData.newPlainText("cutText", textStr)
        clipboard.setPrimaryClip(clip)
        textEditor.setText("")
        showToast("Text cut to clipboard!")
    end
end

hindiSpeechButton.onClick = function()
    if hindiSpeechActive then
        stopTranscription()
        hindiSpeechActive = false
    else
        toggleTranscriptionMode("hi-IN")
    end
end

englishSpeechButton.onClick = function()
    if englishSpeechActive then
        stopTranscription()
        englishSpeechActive = false
    else
        toggleTranscriptionMode("en-US")
    end
end

analyzeTextButton.onClick = showTextAnalysis
